@@ -1,33 +1,45 @@
 #!/usr/bin/env python3
 
+from typing import Optional, List
+
 import hmac
 import hashlib
 
 from ..libs.segwit import encode
-from ..libs.ed25519 import (encodepoint, decodepoint, decodeint, scalarmultbase, edwards_add)
-from .utils import prune_intermediate_scalar, get_bytes, bad_seed_checker
+from ..libs.ed25519 import (
+    encodepoint, decodepoint, decodeint, scalarmultbase, edwards_add
+)
+from .utils import (
+    prune_intermediate_scalar, get_bytes, bad_seed_checker
+)
+from ..exceptions import NetworkError
+from ..utils import is_network
+from ..config import config
 
 
+# Bytom config
+config: dict = config()
 # Constant values
-HARDEN = 0x80000000
+HARDEN: int = 0x80000000
 # Derivation Path
-PATH = "m/44/153/1/0/1"
+PATH: str = "m/44/153/1/0/1"
 # Derivation Indexes
-INDEXES = [
-    "2c000000",  # 44
-    "99000000",  # 153
-    "01000000",  # 1 Account
-    "00000000",  # 0 Change
-    "01000000"  # 1 Address
+INDEXES: List[str] = [
+    # "2c000000",  # 44
+    # "99000000",  # 153
+    # "01000000",  # 1 Account
+    # "00000000",  # 0 Change
+    # "01000000"  # 1 Address
 ]
 
 
-def get_xpublic_key(xprivate_key):
+def get_xpublic_key(xprivate_key: str) -> str:
     """
     Get Bytom xpublic key from xprivate key.
 
     :param xprivate_key: Bytom xprivate key.
     :type xprivate_key: str.
+
     :return: str -- Bytom xpublic key.
 
     >>> from pybytom.wallet.tools import get_xpublic_key
@@ -35,10 +47,6 @@ def get_xpublic_key(xprivate_key):
     "16476b7fd68ca2acd92cfc38fa353e75d6103f828276f44d587e660a6bd7a5c5ef4490504bd2b6f997113671892458830de09518e6bd5958d5d5dd97624cfa4b"
     """
 
-    # Checking parameters
-    if not isinstance(xprivate_key, str):
-        raise TypeError("xprivate key must be string format")
-
     xprivate_bytes = get_bytes(xprivate_key)
     scalar = decodeint(xprivate_bytes[:len(xprivate_bytes) // 2])
     buf = encodepoint(scalarmultbase(scalar))
@@ -46,12 +54,13 @@ def get_xpublic_key(xprivate_key):
     return xpublic_key.hex()
 
 
-def get_expand_xprivate_key(xprivate_key):
+def get_expand_xprivate_key(xprivate_key: str) -> str:
     """
     Get Bytom expand xprivate key from xprivate key.
 
     :param xprivate_key: Bytom xprivate key.
     :type xprivate_key: str.
+
     :return: str -- Bytom expand xprivate key.
 
     >>> from pybytom.wallet.tools import get_expand_xprivate_key
@@ -59,10 +68,6 @@ def get_expand_xprivate_key(xprivate_key):
     "205b15f70e253399da90b127b074ea02904594be9d54678207872ec1ba31ee5102416c643cfb46ab1ae5a524c8b4aaa002eb771d0d9cfc7490c0c3a8177e053e"
     """
 
-    # Checking parameters
-    if not isinstance(xprivate_key, str):
-        raise TypeError("xprivate key must be string format")
-
     i = hmac.HMAC(b"Expand", get_bytes(xprivate_key),
                   digestmod=hashlib.sha512).hexdigest()
     il, ir = i[:64], i[64:]
@@ -72,12 +77,13 @@ def get_expand_xprivate_key(xprivate_key):
     return expand_xprivate_key
 
 
-def indexes_to_path(indexes):
+def indexes_to_path(indexes: List[str]) -> str:
     """
     Change derivation indexes to path.
 
     :param indexes: Bytom derivation indexes.
     :type indexes: list.
+
     :return: str -- Bytom derivation path.
 
     >>> from pybytom.wallet.tools import indexes_to_path
@@ -85,10 +91,6 @@ def indexes_to_path(indexes):
     "m/44/153/1/0/1"
     """
 
-    # Checking parameters
-    if not isinstance(indexes, list):
-        raise TypeError("indexes must be list format")
-
     path = "m/"
     for i, index in enumerate(indexes, 1):
         number = int.from_bytes(bytes.fromhex(index), byteorder="little")
@@ -99,12 +101,13 @@ def indexes_to_path(indexes):
     return path
 
 
-def path_to_indexes(path):
+def path_to_indexes(path: str) -> List[str]:
     """
     Change derivation path to indexes.
 
     :param path: Bytom derivation path.
     :type path: str.
+
     :return: list -- Bytom derivation indexes.
 
     >>> from pybytom.wallet.tools import path_to_indexes
@@ -112,9 +115,6 @@ def path_to_indexes(path):
     ["2c000000", "99000000", "01000000", "00000000", "01000000"]
     """
 
-    # Checking parameters
-    if not isinstance(path, str):
-        raise TypeError("path must be string format")
     if str(path)[0:2] != "m/":
         raise ValueError("bad path, insert like this type of path \"m/0'/0\"! ")
 
@@ -129,7 +129,8 @@ def path_to_indexes(path):
     return indexes
 
 
-def get_child_xprivate_key(xprivate_key, indexes=None, path=None):
+def get_child_xprivate_key(xprivate_key: str, indexes: Optional[List[str]] = None,
+                           path: Optional[str] = None) -> str:
     """
     Get Bytom get child xprivate key.
 
@@ -139,6 +140,7 @@ def get_child_xprivate_key(xprivate_key, indexes=None, path=None):
     :type indexes: list.
     :param path: Bytom derivation path, default to None.
     :type path: str.
+
     :return: str -- Bytom child xprivate key.
 
     >>> from pybytom.wallet.tools import get_child_xprivate_key
@@ -146,17 +148,9 @@ def get_child_xprivate_key(xprivate_key, indexes=None, path=None):
     "00ca8555655d336c4c0d11464a1d401f0cc7c29fdc52bf52f5fc8e0ced32ee51b9c62d145693b366cde5ba74a06962bfa9f6b1e810a3e15eadf791247333547e"
     """
 
-    # Checking parameters
-    if not isinstance(xprivate_key, str):
-        raise TypeError("xprivate key must be string format")
     if indexes is None and path is None:
         indexes = INDEXES
-    elif indexes is not None:
-        if not isinstance(indexes, list):
-            raise TypeError("indexes must be list format")
     elif path is not None:
-        if not isinstance(path, str):
-            raise TypeError("path must be string format")
         indexes = path_to_indexes(path=path)
 
     for index in range(len(indexes)):
@@ -186,7 +180,8 @@ def get_child_xprivate_key(xprivate_key, indexes=None, path=None):
     return child_xprivate
 
 
-def get_child_xpublic_key(xpublic_key, indexes=None, path=None):
+def get_child_xpublic_key(xpublic_key, indexes: Optional[List[str]] = None,
+                          path: Optional[str] = None) -> str:
     """
     Get Bytom get child xpublic key.
 
@@ -196,6 +191,7 @@ def get_child_xpublic_key(xpublic_key, indexes=None, path=None):
     :type indexes: list.
     :param path: Bytom derivation path, default to None.
     :type path: str.
+
     :return: str -- Bytom child xpublic key.
 
     >>> from pybytom.wallet.tools import get_child_xpublic_key
@@ -203,17 +199,9 @@ def get_child_xpublic_key(xpublic_key, indexes=None, path=None):
     "91ff7f525ff40874c4f47f0cab42e46e3bf53adad59adef9558ad1b6448f22e25803ee0a6682fb19e279d8f4f7acebee8abd0fc74771c71565f9a9643fd77141"
     """
 
-    # Checking parameters
-    if not isinstance(xpublic_key, str):
-        raise TypeError("xpublic key must be string format")
     if indexes is None and path is None:
         indexes = INDEXES
-    elif indexes is not None:
-        if not isinstance(indexes, list):
-            raise TypeError("indexes must be list format")
     elif path is not None:
-        if not isinstance(path, str):
-            raise TypeError("path must be string format")
         indexes = path_to_indexes(path=path)
 
     for index in range(len(indexes)):
@@ -241,7 +229,8 @@ def get_child_xpublic_key(xpublic_key, indexes=None, path=None):
     return child_xpublic
 
 
-def get_private_key(xprivate_key, indexes=None, path=None):
+def get_private_key(xprivate_key: str, indexes: Optional[List[str]] = None,
+                    path: Optional[str] = None) -> str:
     """
     Get Bytom private key from xprivate key. This is also the same with get_child_xprivate_key function.
 
@@ -251,6 +240,7 @@ def get_private_key(xprivate_key, indexes=None, path=None):
     :type indexes: list.
     :param path: Bytom derivation path, default to None.
     :type path: str.
+
     :return: str -- Bytom private key.
 
     >>> from pybytom.wallet.tools import get_private_key
@@ -262,7 +252,8 @@ def get_private_key(xprivate_key, indexes=None, path=None):
         xprivate_key=xprivate_key, indexes=indexes, path=path)
 
 
-def get_public_key(xpublic_key=None, indexes=None, path=None):
+def get_public_key(xpublic_key: str, indexes: Optional[List[str]] = None,
+                   path: Optional[str] = None) -> str:
     """
     Get Bytom public key from xpublic key.
 
@@ -272,6 +263,7 @@ def get_public_key(xpublic_key=None, indexes=None, path=None):
     :type indexes: list.
     :param path: Bytom derivation path, default to None.
     :type path: str.
+
     :return: str -- Bytom public key.
 
     >>> from pybytom.wallet.tools import get_public_key
@@ -283,7 +275,7 @@ def get_public_key(xpublic_key=None, indexes=None, path=None):
             xpublic_key=xpublic_key, indexes=indexes, path=path)[:64]
 
 
-def get_program(public_key):
+def get_program(public_key: str) -> str:
     """
     Get Bytom control program from public key.
 
@@ -296,10 +288,6 @@ def get_program(public_key):
     "00142cda4f99ea8112e6fa61cdd26157ed6dc408332a"
     """
 
-    # Checking parameters
-    if not isinstance(public_key, str):
-        raise TypeError("public key must be string format")
-
     public_byte = get_bytes(public_key)
     ripemd160 = hashlib.new("ripemd160")
     ripemd160.update(public_byte)
@@ -308,14 +296,15 @@ def get_program(public_key):
     return control_program
 
 
-def get_address(program, network="solonet"):
+def get_address(program: str, network: str = config["network"]) -> str:
     """
     Get Bytom address from program.
 
     :param program: Bytom control program.
     :type program: str.
-    :param network: Bytom network, default to solonet.
+    :param network: Bytom network, default to mainnet.
     :type network: str.
+
     :return: str -- Bytom address.
 
     >>> from pybytom.wallet.tools import get_address
@@ -323,13 +312,9 @@ def get_address(program, network="solonet"):
     "bm1q9ndylx02syfwd7npehfxz4lddhzqsve2fu6vc7"
     """
 
-    if not isinstance(program, str):
-        raise TypeError("program must be string format")
-    if not isinstance(network, str):
-        raise TypeError("network must be string format")
-
-    if network not in "mainnet/solonet/testnet".split("/"):
-        raise ValueError("invalid network option, choose only mainnet, solonet and testnet network")
+    if not is_network(network=network):
+        raise NetworkError(f"Invalid '{network}' network",
+                           "choose only 'mainnet', 'solonet' or 'testnet' networks.")
     elif network == "mainnet":
         return encode("bm", 0, get_bytes(program[4:]))
     elif network == "solonet":
@@ -338,7 +323,7 @@ def get_address(program, network="solonet"):
         return encode("tm", 0, get_bytes(program[4:]))
 
 
-def get_vapor_address(program, network="solonet"):
+def get_vapor_address(program: str, network: str = config["network"]) -> str:
     """
     Get Bytom vapor address from program.
 
@@ -346,6 +331,7 @@ def get_vapor_address(program, network="solonet"):
     :type program: str.
     :param network: Bytom network, default to solonet.
     :type network: str.
+
     :return: str -- Bytom vapor address.
 
     >>> from pybytom.wallet.tools import get_vapor_address
@@ -353,13 +339,9 @@ def get_vapor_address(program, network="solonet"):
     "vp1q9ndylx02syfwd7npehfxz4lddhzqsve2za23ag"
     """
 
-    if not isinstance(program, str):
-        raise TypeError("program must be string format")
-    if not isinstance(network, str):
-        raise TypeError("network must be string format")
-
-    if network not in "mainnet/solonet/testnet".split("/"):
-        raise ValueError("invalid network option, choose only mainnet, solonet and testnet network")
+    if not is_network(network=network):
+        raise NetworkError(f"Invalid '{network}' network",
+                           "choose only 'mainnet', 'solonet' or 'testnet' networks.")
     elif network == "mainnet":
         return encode("vp", 0, get_bytes(program[4:]))
     elif network == "solonet":
